@@ -30,11 +30,13 @@ typedef struct {                         // type of structure for a record
 const char *g_scriptname;                // script name
 const char *g_scriptprefix;              // script prefix
 const char *g_scriptpath;                // script path
+const char *g_docroot;                   // document root
 TCTMPL *g_tmpl;                          // template serializer
 const char *g_database;                  // path of the database file
 const char *g_password;                  // path of the password file
 TCMAP *g_users;                          // user list
 const char *g_upload;                    // path of the upload directory
+const char *g_uploadpub;                 // public path of the upload directory
 int64_t g_recvmax;                       // maximum size of received data
 const char *g_title;                     // site title
 int g_searchnum;                         // number of articles in a search page
@@ -79,6 +81,18 @@ int main(int argc, char **argv){
   g_scriptprefix = prefix;
   g_scriptpath = getenv("SCRIPT_FILENAME");
   if(!g_scriptpath) g_scriptpath = argv[0];
+  g_docroot = getenv("DOCUMENT_ROOT");
+  char *docrootbuf = NULL;
+  if(!g_docroot){
+    g_docroot = "/";
+    if(*g_scriptpath == '/'){
+      int diff = strlen(g_scriptpath) - strlen(g_scriptname);
+      if(diff > 0 && !strcmp(g_scriptpath + diff, g_scriptname)){
+        docrootbuf = tcmemdup(g_scriptpath, diff);
+        g_docroot = docrootbuf;
+      }
+    }
+  }
   char *barepath = tcregexreplace(g_scriptpath, "\\.[a-zA-Z0-9]*$", "");
   char *tmplpath = tcsprintf("%s.tmpl", barepath);
   g_tmpl = tctmplnew();
@@ -92,6 +106,21 @@ int main(int argc, char **argv){
       readpasswd();
     }
     g_upload = tctmplconf(g_tmpl, "upload");
+    g_uploadpub = NULL;
+    char *uploadpubbuf = NULL;
+    if(g_upload && *g_upload != '\0'){
+      if(!strchr(g_upload, '/')){
+        g_uploadpub = g_upload;
+      } else {
+        uploadpubbuf = tcrealpath(g_upload);
+        if(uploadpubbuf){
+          if(tcstrfwm(uploadpubbuf, g_docroot)){
+            int plen = strlen(g_docroot);
+            if(uploadpubbuf[plen] == '/') g_uploadpub = uploadpubbuf + plen;
+          }
+        }
+      }
+    }
     const char *rp = tctmplconf(g_tmpl, "recvmax");
     g_recvmax = rp ? tcatoix(rp) : INT_MAX;
     g_title = tctmplconf(g_tmpl, "title");
@@ -124,10 +153,12 @@ int main(int argc, char **argv){
       tcfree(name);
     }
     eventloop();
+    tcfree(uploadpubbuf);
     if(g_users) tcmapdel(g_users);
   } else {
     showerror(500, "The template file is missing.");
   }
+  tcfree(docrootbuf);
   tctmpldel(g_tmpl);
   tcfree(tmplpath);
   tcfree(barepath);
@@ -1138,7 +1169,7 @@ static void dosession(TCMPOOL *mpool){
         tcmapput2(comment, "owner", com->owner);
         tcmapput2(comment, "text", com->text);
         TCXSTR *xstr = tcmpoolxstrnew(mpool);
-        wikitohtmlinline(xstr, com->text, g_scriptname);
+        wikitohtmlinline(xstr, com->text, g_scriptname, g_uploadpub);
         tcmapput(comment, "texthtml", 8, tcxstrptr(xstr), tcxstrsize(xstr));
         tclistpushmap(comments, comment);
       }
@@ -1155,6 +1186,7 @@ static void dosession(TCMPOOL *mpool){
   tcmapput2(vars, "scriptprefix", g_scriptprefix);
   tcmapput2(vars, "scriptpath", g_scriptpath);
   tcmapput2(vars, "scripturl", p_scripturl);
+  tcmapput2(vars, "documentroot", g_docroot);
   if(g_users) tcmapputmap(vars, "users", g_users);
   if(auth && p_user != '\0'){
     tcmapput2(vars, "username", p_user);
@@ -1163,6 +1195,7 @@ static void dosession(TCMPOOL *mpool){
   if(auth) tcmapput2(vars, "auth", "true");
   if(admin) tcmapput2(vars, "admin", "true");
   if(cancom) tcmapput2(vars, "cancom", "true");
+  if(g_uploadpub) tcmapput2(vars, "uploadpub", g_uploadpub);
   tcmapput2(vars, "tpversion", TPVERSION);
   char numbuf[NUMBUFSIZ];
   tcdatestrwww(now, INT_MAX, numbuf);
@@ -1258,7 +1291,7 @@ static void setarthtml(TCMPOOL *mpool, TCMAP *cols, int64_t id, int bhl, bool ti
       tcmapput2(cols, "texttiny", str);
     } else {
       TCXSTR *xstr = tcmpoolxstrnew(mpool);
-      wikitohtml(xstr, rp, idbuf, g_scriptname, bhl + 1);
+      wikitohtml(xstr, rp, idbuf, g_scriptname, bhl + 1, g_uploadpub);
       if(tcxstrsize(xstr) > 0) tcmapput(cols, "texthtml", 8, tcxstrptr(xstr), tcxstrsize(xstr));
     }
   }
@@ -1287,7 +1320,7 @@ static void setarthtml(TCMPOOL *mpool, TCMAP *cols, int64_t id, int bhl, bool ti
           tcmapput2(comment, "owner", co);
           tcmapput2(comment, "text", ct);
           TCXSTR *xstr = tcmpoolxstrnew(mpool);
-          wikitohtmlinline(xstr, ct, g_scriptname);
+          wikitohtmlinline(xstr, ct, g_scriptname, g_uploadpub);
           tcmapput(comment, "texthtml", 8, tcxstrptr(xstr), tcxstrsize(xstr));
           tclistpushmap(comments, comment);
         }
